==> Demo_CH32F103C8T6_ST7789V2_3-Line_Hardware_SPI.zip: Demo_CH32F103C8T6_ST7789V2_3-Line_Hardware_SPI/HARDWARE/FLASH/flash.c <==
/**********************************************************************
 * ֻѧϰʹãδɣκ;
 * ӲƬCH32F103C8T6,Ƶ72MHZ
 * ˵ST7789V 3ӲSPI
 
 ----------------------------------------------------
 |ǧһʾƼ޹˾                        |
 |˾վ:www.qytft.com                            |
 |Ավhttps://shop580133616.taobao.com        |
 |̻() :+86 0755-27908130                     |
 |ҵѯ:18823372746 (쾭 WXͬ)              |
 |ҵѯ:15989313508 (뾭 WXͬ)              |
 ----------------------------------------------------

 ----------------------------------------------------
 |˾ṩ֧֣κμ⻶ӭʱѧϰ    |
 |֧QQ:3597321058                             |
 |֧:qyxskj@126.com                       |
 |֧:13723763193ܹ WXͬţ               |
 |QQȺ:792027219                            |
 ----------------------------------------------------

 * ˵:
 ========================ԴŽ==========================================        
 |   LCDģ           Ƭ                                             |
 |     VCC      --->      3.3V      //Դ                                    |
 |     GND      --->      GND       //Դ                                  | 
 |====================ƼŽ======================================|
 |   LCDģ 			     Ƭ                                             |
 |     CS       --->      PA4       //ҺƬѡźţ͵ƽЧ        |
 |     RST      --->      PA1       //Һλźţ͵ƽЧ        |
 |     SCL      --->      PA5       //ҺSPIʱź                   |
 |     SDI      --->      PA7       //ҺSPIдź                 |
 |     SDO      --->      PA6       //ҺSPI߶ź                 |
 |     LED      --->      PA15      //Һźţ粻ƣ3.3V|
 |================================================================|
 |	 LCDģ 			     Ƭ                                             |
 |     T_IRQ    --->      PA12      //жź                      |
 |     T_DO     --->      PA2       //SPI߶ź                     |
 |     T_DIN    --->      PA3       //SPIдź                     |
 |     T_CS     --->      PA11      //Ƭѡź                      |
 |     T_CLK    --->      PA8       //SPIʱź                   |
 ==============================================================================
 
 * :2020/5/13
 * 汾V1.0
 * ȨУؾ
 * Copyright(C) ǧһʾƼ޹˾ 2020-2035
 * All rights reserved
**********************************************************************/		
#include "flash.h"
#include "debug.h"

#if FLASH_WREN	//ʹд   

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :unchecked to write to flash
 * @parameters :WriteAddr:the start address to be written of flash 
								pBuffer:the point of written data
								NumToWrite:the number of halfword to be written
 * @retvalue   :None
******************************************************************************/   
void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite)   
{ 			 		 
	u16 i;
	for(i=0;i<NumToWrite;i++)
	{
		FLASH_ProgramHalfWord(WriteAddr,pBuffer[i]);
	    WriteAddr+=2;//ַ2.
	}  
} 

#if FLASH_SIZE<256
#define FLASH_SECTOR_SIZE 1024 //ֽ
#else 
#define FLASH_SECTOR_SIZE	2048
#endif		 
u16 FLASH_BUF[FLASH_SECTOR_SIZE/2];//2Kֽ

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :Write data of the specified length from the specified address
 * @parameters :WriteAddr:the start address to be written of flash 
								pBuffer:the point of written data
								NumToWrite:the number of halfword to be written
 * @retvalue   :None
******************************************************************************/  
void FLASH_Write(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite)	
{
	u32 secpos;	   //ַ
	u16 secoff;	   //ƫƵַ(16λּ)
	u16 secremain; //ʣַ(16λּ)	   
 	u16 i;    
	u32 offaddr;   //ȥ0X08000000ĵַ
	if(WriteAddr<FLASH_BASE_ADDRESS||(WriteAddr>=(FLASH_BASE_ADDRESS+1024*FLASH_SIZE)))return;//Ƿַ
	FLASH_Unlock();						//
	offaddr=WriteAddr-FLASH_BASE_ADDRESS;		//ʵƫƵַ.
	secpos=offaddr/FLASH_SECTOR_SIZE;			//ַ  0~127 for STM32F103RBT6
	secoff=(offaddr%FLASH_SECTOR_SIZE)/2;		//ڵƫ(2ֽΪλ.)
	secremain=FLASH_SECTOR_SIZE/2-secoff;		//ʣռС   
	if(NumToWrite<=secremain)secremain=NumToWrite;//ڸΧ
	while(1) 
	{	
		FLASH_Read(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS,FLASH_BUF,FLASH_SECTOR_SIZE/2);//
		for(i=0;i<secremain;i++)//У
		{
			if(FLASH_BUF[secoff+i]!=0XFFFF)break;//Ҫ  	  
		}
		if(i<secremain)//Ҫ
		{
			FLASH_ErasePage(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS);//
			for(i=0;i<secremain;i++)//
			{
				FLASH_BUF[i+secoff]=pBuffer[i];	  
			}
			FLASH_Write_NoCheck(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS,FLASH_BUF,FLASH_SECTOR_SIZE/2);//д  
		}else FLASH_Write_NoCheck(WriteAddr,pBuffer,secremain);//дѾ˵,ֱдʣ. 				   
		if(NumToWrite==secremain)break;//д
		else//дδ
		{
			secpos++;				//ַ1
			secoff=0;				//ƫλΪ0 	 
		   	pBuffer+=secremain;  	//ָƫ
			WriteAddr+=secremain;	//дַƫ	   
		   	NumToWrite-=secremain;	//ֽ(16λ)ݼ
			if(NumToWrite>(FLASH_SECTOR_SIZE/2))secremain=FLASH_SECTOR_SIZE/2;//һд
			else secremain=NumToWrite;//һд
		}	 
	};	
	FLASH_Lock();//
}
#endif

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :Read data of the specified length from the specified address
 * @parameters :WriteAddr:the start address to be read of flash 
								pBuffer:the point of read data
								NumToWrite:the number of halfword to be read
 * @retvalue   :None
******************************************************************************/  
void FLASH_Read(u32 ReadAddr,u16 *pBuffer,u16 NumToRead)   	
{
	u16 i;
	for(i=0;i<NumToRead;i++)
	{
		pBuffer[i]=*(vu16*)ReadAddr;//ȡ2ֽ.
		ReadAddr+=2;//ƫ2ֽ.	
	}
}

==> Demo_CH32F103C8T6_ST7789V2_4-Line_Hardware_SPI.zip: Demo_CH32F103C8T6_ST7789V2_4-Line_Hardware_SPI/HARDWARE/FLASH/flash.c <==
/**********************************************************************
 * ֻѧϰʹãδɣκ;
 * ӲƬCH32F103C8T6,Ƶ72MHZ
 * ˵ST7789V 4ӲSPI
 
 ----------------------------------------------------
 |ǧһʾƼ޹˾                        |
 |˾վ:www.qytft.com                            |
 |Ավhttps://shop580133616.taobao.com        |
 |̻() :+86 0755-27908130                     |
 |ҵѯ:18823372746 (쾭 WXͬ)              |
 |ҵѯ:15989313508 (뾭 WXͬ)              |
 ----------------------------------------------------

 ----------------------------------------------------
 |˾ṩ֧֣κμ⻶ӭʱѧϰ    |
 |֧QQ:3597321058                             |
 |֧:qyxskj@126.com                       |
 |֧:13723763193ܹ WXͬţ               |
 |QQȺ:792027219                            |
 ----------------------------------------------------

 * ˵:
 ========================ԴŽ=======================================================        
 |   LCDģ           Ƭ                                                          |
 |     VCC      --->      3.3V      //Դ                                                 |
 |     GND      --->      GND       //Դ                                               |  
 |====================ƼŽ===================================================|
 |   LCDģ 			     Ƭ                                                          |
 |     CS       --->      PA4       //ҺƬѡźţ͵ƽЧ                     |
 |     RST      --->      PA1       //Һλźţ͵ƽЧ                     |
 |     RS       --->      PA10      //Һ/ݿźţ͵ƽߵƽݣ|
 |     SCL      --->      PA5       //ҺSPIʱź                                |
 |     SDI      --->      PA7       //ҺSPIдź                              |
 |     SDO      --->      PA6       //ҺSPI߶ź                              |
 |     LED      --->      PA15      //Һźţ粻ƣ3.3V             |
 |=============================================================================|
 |	 LCDģ 			     Ƭ                                                          |
 |     T_IRQ    --->      PA12      //жź                                   |
 |     T_DO     --->      PA2       //SPI߶ź                                  |
 |     T_DIN    --->      PA3       //SPIдź                                  |
 |     T_CS     --->      PA11      //Ƭѡź                                   |
 |     T_CLK    --->      PA8       //SPIʱź                                |
 ===========================================================================================
 
 * :2020/5/13
 * 汾V1.0
 * ȨУؾ
 * Copyright(C) ǧһʾƼ޹˾ 2020-2035
 * All rights reserved
**********************************************************************/		
#include "flash.h"
#include "debug.h"

#if FLASH_WREN	//ʹд   

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :unchecked to write to flash
 * @parameters :WriteAddr:the start address to be written of flash 
								pBuffer:the point of written data
								NumToWrite:the number of halfword to be written
 * @retvalue   :None
******************************************************************************/   
void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite)   
{ 			 		 
	u16 i;
	for(i=0;i<NumToWrite;i++)
	{
		FLASH_ProgramHalfWord(WriteAddr,pBuffer[i]);
	    WriteAddr+=2;//ַ2.
	}  
} 

#if FLASH_SIZE<256
#define FLASH_SECTOR_SIZE 1024 //ֽ
#else 
#define FLASH_SECTOR_SIZE	2048
#endif		 
u16 FLASH_BUF[FLASH_SECTOR_SIZE/2];//2Kֽ

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :Write data of the specified length from the specified address
 * @parameters :WriteAddr:the start address to be written of flash 
								pBuffer:the point of written data
								NumToWrite:the number of halfword to be written
 * @retvalue   :None
******************************************************************************/  
void FLASH_Write(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite)	
{
	u32 secpos;	   //ַ
	u16 secoff;	   //ƫƵַ(16λּ)
	u16 secremain; //ʣַ(16λּ)	   
 	u16 i;    
	u32 offaddr;   //ȥ0X08000000ĵַ
	if(WriteAddr<FLASH_BASE_ADDRESS||(WriteAddr>=(FLASH_BASE_ADDRESS+1024*FLASH_SIZE)))return;//Ƿַ
	FLASH_Unlock();						//
	offaddr=WriteAddr-FLASH_BASE_ADDRESS;		//ʵƫƵַ.
	secpos=offaddr/FLASH_SECTOR_SIZE;			//ַ  0~127 for STM32F103RBT6
	secoff=(offaddr%FLASH_SECTOR_SIZE)/2;		//ڵƫ(2ֽΪλ.)
	secremain=FLASH_SECTOR_SIZE/2-secoff;		//ʣռС   
	if(NumToWrite<=secremain)secremain=NumToWrite;//ڸΧ
	while(1) 
	{	
		FLASH_Read(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS,FLASH_BUF,FLASH_SECTOR_SIZE/2);//
		for(i=0;i<secremain;i++)//У
		{
			if(FLASH_BUF[secoff+i]!=0XFFFF)break;//Ҫ  	  
		}
		if(i<secremain)//Ҫ
		{
			FLASH_ErasePage(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS);//
			for(i=0;i<secremain;i++)//
			{
				FLASH_BUF[i+secoff]=pBuffer[i];	  
			}
			FLASH_Write_NoCheck(secpos*FLASH_SECTOR_SIZE+FLASH_BASE_ADDRESS,FLASH_BUF,FLASH_SECTOR_SIZE/2);//д  
		}else FLASH_Write_NoCheck(WriteAddr,pBuffer,secremain);//дѾ˵,ֱдʣ. 				   
		if(NumToWrite==secremain)break;//д
		else//дδ
		{
			secpos++;				//ַ1
			secoff=0;				//ƫλΪ0 	 
		   	pBuffer+=secremain;  	//ָƫ
			WriteAddr+=secremain;	//дַƫ	   
		   	NumToWrite-=secremain;	//ֽ(16λ)ݼ
			if(NumToWrite>(FLASH_SECTOR_SIZE/2))secremain=FLASH_SECTOR_SIZE/2;//һд
			else secremain=NumToWrite;//һд
		}	 
	};	
	FLASH_Lock();//
}
#endif

/*****************************************************************************
 * @name       :void FLASH_Write_NoCheck(u32 WriteAddr,u16 *pBuffer,u16 NumToWrite) 
 * @date       :2020-03-02 
 * @function   :Read data of the specified length from the specified address
 * @parameters :WriteAddr:the start address to be read of flash 
								pBuffer:the point of read data
								NumToWrite:the number of halfword to be read
 * @retvalue   :None
******************************************************************************/  
void FLASH_Read(u32 ReadAddr,u16 *pBuffer,u16 NumToRead)   	
{
	u16 i;
	for(i=0;i<NumToRead;i++)
	{
		pBuffer[i]=*(vu16*)ReadAddr;//ȡ2ֽ.
		ReadAddr+=2;//ƫ2ֽ.	
	}
}
